@@ -5,6 +5,7 @@
 #include "qbar3dseries_p.h"
 #include "qbardataproxy_p.h"
 #include "qcategory3daxis_p.h"
+#include "qgraphsinputhandler_p.h"
 #include "qquickgraphsbars_p.h"
 #include "qquickgraphstexturedata_p.h"
 #include "qvalue3daxis_p.h"
@@ -24,7 +25,6 @@ QQuickGraphsBars::QQuickGraphsBars(QQuickItem *parent)
     setAxisZ(0);
     setAcceptedMouseButtons(Qt::AllButtons);
     setFlags(ItemHasContents);
-    createInitialInputHandler();
     clearSelection();
 }
 
@@ -380,12 +380,6 @@ void QQuickGraphsBars::handleSeriesVisibilityChangedBySender(QObject *sender)
 {
     QQuickGraphsItem::handleSeriesVisibilityChangedBySender(sender);
 
-    QBar3DSeries *series = static_cast<QBar3DSeries *>(sender);
-    if (selectionMode().testFlag(QAbstract3DGraph::SelectionMultiSeries) && !series->isVisible()
-        && m_selectedBarSeries == series) {
-        resetClickedStatus();
-    }
-
     // Visibility changes may require disabling slicing,
     // so just reset selection to ensure everything is still valid.
     setSelectedBar(m_selectedBar, m_selectedBarSeries, false);
@@ -442,7 +436,7 @@ void QQuickGraphsBars::adjustAxisRanges()
                     }
 
                     if (adjustX && proxy) {
-                        const QBarDataArray &array = proxy->array();
+                        const QBarDataArray &array = barSeries->dataArray();
                         int columnCount = 0;
                         for (int i = 0; i < array.size(); i++) {
                             if (columnCount < array.at(i).size())
@@ -551,6 +545,8 @@ void QQuickGraphsBars::componentComplete()
                      &QQuick3DNode::rotationChanged,
                      this,
                      &QQuickGraphsBars::handleCameraRotationChanged);
+
+    graphsInputHandler()->setGraphsItem(this);
 }
 
 void QQuickGraphsBars::synchData()
@@ -587,7 +583,6 @@ void QQuickGraphsBars::synchData()
 
     // Do not clear dirty flag, we need to react to it in qquickgraphicsitem as well
     if (theme()->d_func()->m_dirtyBits.backgroundEnabledDirty) {
-        m_floorBackground->setVisible(theme()->isBackgroundEnabled());
         setSeriesVisualsDirty(true);
         for (auto it = m_barModelsMap.begin(); it != m_barModelsMap.end(); it++)
             it.key()->d_func()->m_changeTracker.meshChanged = true;
@@ -606,10 +601,16 @@ void QQuickGraphsBars::synchData()
     QQuickGraphsItem::synchData();
 
     // Draw floor
+
+    //margin for a line to be fully visible on the edge in the grid shader
+    const float halfLineWidth = 50.0;
+    const float gridTextureSize = 4096.0;
+    const float gridMargin = halfLineWidth / gridTextureSize;
     m_floorBackground->setPickable(false);
     auto min = qMin(scaleWithBackground().x(), scaleWithBackground().z());
-    m_floorBackgroundScale->setScale(
-        QVector3D(scaleWithBackground().x(), min * gridOffset(), scaleWithBackground().z()));
+    m_floorBackgroundScale->setScale(QVector3D(scaleWithBackground().x() + gridMargin,
+                                               min * gridOffset(),
+                                               scaleWithBackground().z() + gridMargin));
     m_floorBackgroundScale->setPosition(QVector3D(0.0f, -m_backgroundAdjustment, 0.0f));
 
     QQuaternion m_xRightAngleRotation(QQuaternion::fromAxisAndAngle(1.0f, 0.0f, 0.0f, 90.0f));
@@ -621,19 +622,12 @@ void QQuickGraphsBars::synchData()
         m_floorBackgroundRotation->setRotation(m_xRightAngleRotationNeg);
 
     QQmlListReference materialsRefF(m_floorBackground, "materials");
-    QQuick3DPrincipledMaterial *bgMatFloor;
-    if (!materialsRefF.size()) {
-        bgMatFloor = new QQuick3DPrincipledMaterial();
-        bgMatFloor->setParent(m_floorBackground);
-        bgMatFloor->setMetalness(0.f);
-        bgMatFloor->setRoughness(.3f);
-        bgMatFloor->setEmissiveFactor(QVector3D(.001f, .001f, .001f));
+    QQmlListReference bbRef(background(), "materials");
+    QQuick3DCustomMaterial *bgMatFloor;
+    if (!materialsRefF.size() && bbRef.size()) {
+        bgMatFloor = static_cast<QQuick3DCustomMaterial *>(bbRef.at(0));
         materialsRefF.append(bgMatFloor);
-    } else {
-        bgMatFloor = static_cast<QQuick3DPrincipledMaterial *>(materialsRefF.at(0));
     }
-    bgMatFloor->setBaseColor(theme()->backgroundColor());
-
     if (m_selectedBarPos.isNull())
         itemLabel()->setVisible(false);
 }
@@ -686,24 +680,10 @@ void QQuickGraphsBars::updateGraph()
 
     if (isDataDirty()) {
         removeBarModels();
-        if (sliceView() && sliceView()->isVisible())
-            removeSlicedBarModels();
         generateBars(barSeriesAsList);
     }
 
     if (isSeriesVisualsDirty()) {
-        if (isSliceEnabled()) {
-            removeSlicedBarModels();
-            createSliceView();
-
-            for (const auto &barSeries : std::as_const(barSeriesAsList)) {
-                bool visible = !(sliceView()->isVisible() ^ barSeries->isVisible());
-                if (m_selectedBarSeries == barSeries) {
-                    setSliceActivatedChanged(true);
-                    m_selectionDirty = !visible;
-                }
-            }
-        }
         int visualIndex = 0;
         for (const auto &barSeries : std::as_const(barSeriesAsList)) {
             if (barSeries->isVisible()) {
@@ -715,12 +695,34 @@ void QQuickGraphsBars::updateGraph()
                 updateBarVisuality(barSeries, -1);
             }
         }
+    }
 
-        // Needs to be done after data is set, as it needs to know the visual array.
-        if (m_changeTracker.selectedBarChanged) {
-            updateSelectedBar();
-            m_changeTracker.selectedBarChanged = false;
+    // Needs to be done after data is set, as it needs to know the visual array.
+    if (m_changeTracker.selectedBarChanged) {
+        updateSelectedBar();
+        if (isSliceEnabled()) {
+            createSliceView();
+            for (const auto &series : std::as_const(barSeriesAsList)) {
+                bool visible = (m_selectedBarSeries == series) && series->isVisible();
+                if (sliceView()->isVisible()) {
+                    if (visible) {
+                        removeSlicedBarModels();
+                        createSliceView();
+                        setSliceActivatedChanged(false);
+                        updateSliceGraph();
+                        break;
+                    } else {
+                        setSliceActivatedChanged(true);
+                    }
+                } else {
+                    if (visible) {
+                        m_selectionDirty = true;
+                        setSliceActivatedChanged(true);
+                    }
+                }
+            }
         }
+        m_changeTracker.selectedBarChanged = false;
     }
 
     setDataDirty(false);
@@ -1073,7 +1075,7 @@ void QQuickGraphsBars::handleDataRowLabelsChanged()
         int count = int(m_axisZ->max()) - min + 1;
         QStringList subList;
         if (m_primarySeries && m_primarySeries->dataProxy())
-            subList = m_primarySeries->dataProxy()->rowLabels().mid(min, count);
+            subList = m_primarySeries->rowLabels().mid(min, count);
         static_cast<QCategory3DAxis *>(m_axisZ)->d_func()->setDataLabels(subList);
     }
 }
@@ -1085,11 +1087,8 @@ void QQuickGraphsBars::handleDataColumnLabelsChanged()
         int min = int(m_axisX->min());
         int count = int(m_axisX->max()) - min + 1;
         QStringList subList;
-        if (m_primarySeries && m_primarySeries->dataProxy()) {
-            subList = static_cast<QBarDataProxy *>(m_primarySeries->dataProxy())
-                          ->columnLabels()
-                          .mid(min, count);
-        }
+        if (m_primarySeries && m_primarySeries->dataProxy())
+            subList = m_primarySeries->columnLabels().mid(min, count);
         static_cast<QCategory3DAxis *>(m_axisX)->d_func()->setDataLabels(subList);
     }
 }
@@ -1508,11 +1507,15 @@ void QQuickGraphsBars::updateBarVisuals(QBar3DSeries *series)
     QLinearGradient gradient = series->baseGradient();
 
     if (optimizationHint() == QAbstract3DGraph::OptimizationHint::Legacy) {
+        // Release resources that might not have been deleted even though deleteLater had been set
+        window()->releaseResources();
+
         for (int i = 0; i < barList.count(); i++) {
             QQuick3DModel *model = barList.at(i)->model;
             auto textureData = static_cast<QQuickGraphsTextureData *>(
                 barList.at(i)->texture->textureData());
             textureData->createGradient(series->baseGradient());
+            const bool transparency = textureData->hasTransparency();
             updateItemMaterial(model,
                                useGradient,
                                rangeGradient,
@@ -1522,7 +1525,8 @@ void QQuickGraphsBars::updateBarVisuals(QBar3DSeries *series)
                                          false,
                                          false,
                                          barList.at(i)->texture,
-                                         QColor(Qt::white));
+                                         QColor(Qt::white),
+                                         transparency);
             } else {
                 QList<QColor> rowColors = series->rowColors();
                 if (rowColors.size() == 0) {
@@ -1531,15 +1535,21 @@ void QQuickGraphsBars::updateBarVisuals(QBar3DSeries *series)
                     int rowColorIndex = barList.at(i)->coord.x() % rowColors.size();
                     barColor = rowColors[rowColorIndex];
                 }
-                updateMaterialProperties(model, false, false, barList.at(i)->texture, barColor);
+                const bool transparency = barColor.alphaF() < 1.0;
+                updateMaterialProperties(model,
+                                         false,
+                                         false,
+                                         barList.at(i)->texture,
+                                         barColor,
+                                         transparency);
             }
         }
     } else if (optimizationHint() == QAbstract3DGraph::OptimizationHint::Default) {
         for (int i = 0; i < barList.count(); i++) {
-            barList.at(i)->instancing->setRangeGradient(rangeGradient);
             auto textureData = static_cast<QQuickGraphsTextureData *>(
                 barList.at(i)->texture->textureData());
             textureData->createGradient(series->baseGradient());
+            barList.at(i)->instancing->setTransparency(textureData->hasTransparency());
             updateItemMaterial(barList.at(i)->model,
                                useGradient,
                                rangeGradient,
@@ -1548,7 +1558,8 @@ void QQuickGraphsBars::updateBarVisuals(QBar3DSeries *series)
                                      false,
                                      false,
                                      barList.at(i)->texture,
-                                     QColor(Qt::white));
+                                     QColor(Qt::white),
+                                     textureData->hasTransparency());
         }
     }
 }
@@ -1588,15 +1599,17 @@ void QQuickGraphsBars::updateItemMaterial(QQuick3DModel *item,
 }
 
 void QQuickGraphsBars::updateMaterialProperties(QQuick3DModel *item,
-                                                bool isHighlight,
-                                                bool isMultiHighlight,
+                                                const bool isHighlight,
+                                                const bool isMultiHighlight,
                                                 QQuick3DTexture *texture,
-                                                const QColor &color)
+                                                const QColor &color,
+                                                const bool transparency)
 {
     QQmlListReference materialsRef(item, "materials");
     auto customMaterial = qobject_cast<QQuick3DCustomMaterial *>(materialsRef.at(0));
     if (!customMaterial)
         return;
+    customMaterial->setProperty("transparency", transparency);
     QVariant textureInputAsVariant = customMaterial->property("custex");
     QQuick3DShaderUtilsTextureInput *textureInput = textureInputAsVariant
                                                         .value<QQuick3DShaderUtilsTextureInput *>();
@@ -1670,34 +1683,6 @@ QQuick3DTexture *QQuickGraphsBars::createTexture()
     return texture;
 }
 
-bool QQuickGraphsBars::handleMousePressedEvent(QMouseEvent *event)
-{
-    if (!QQuickGraphsItem::handleMousePressedEvent(event))
-        return true;
-
-    createSliceView();
-
-    if (Qt::LeftButton == event->button())
-        doPicking(event->pos());
-
-    return true;
-}
-
-bool QQuickGraphsBars::handleTouchEvent(QTouchEvent *event)
-{
-    if (!QQuickGraphsItem::handleTouchEvent(event))
-        return true;
-
-    createSliceView();
-
-    if (scene()->selectionQueryPosition() != scene()->invalidSelectionPoint()
-        && !event->isUpdateEvent()) {
-        doPicking(event->point(0).position());
-        scene()->setSelectionQueryPosition(scene()->invalidSelectionPoint());
-    }
-    return true;
-}
-
 bool QQuickGraphsBars::doPicking(const QPointF &position)
 {
     if (!QQuickGraphsItem::doPicking(position))
@@ -1746,6 +1731,8 @@ bool QQuickGraphsBars::doPicking(const QPointF &position)
                                                 setSelectedBar(bih->coord,
                                                                m_barModelsMap.key(barlist),
                                                                false);
+                                                if (isSliceEnabled())
+                                                    setSliceActivatedChanged(true);
                                             }
                                         }
                                     }
@@ -1961,10 +1948,10 @@ void QQuickGraphsBars::updateSliceItemLabel(QString label, const QVector3D &posi
         slicePos.setX(slicePos.x());
     QValue3DAxis *valueAxis = static_cast<QValue3DAxis *>(axisY());
     if (valueAxis->reversed())
-        slicePos.setY(slicePos.y() - (textPadding * .045f));
+        slicePos.setY(slicePos.y() - (textPadding * .06f));
     else
-        slicePos.setY(slicePos.y() + (textPadding * .045f));
-    slicePos.setZ(1.2f);
+        slicePos.setY(slicePos.y() + (textPadding * .06f));
+    slicePos.setZ(.1f);
     sliceItemLabel()->setPosition(slicePos);
     sliceItemLabel()->setProperty("labelText", label);
     sliceItemLabel()->setEulerRotation(QVector3D(0.0f, 0.0f, 90.0f));
@@ -1995,10 +1982,9 @@ void QQuickGraphsBars::resetClickedStatus()
 
 void QQuickGraphsBars::createSliceView()
 {
+    setSliceOrthoProjection(false);
     QQuickGraphsItem::createSliceView();
 
-    QQuick3DViewport *sliceParent = sliceView();
-
     QList<QBar3DSeries *> barSeries = barSeriesList();
     for (const auto &barSeries : std::as_const(barSeries)) {
         QList<BarModel *> *slicedBarList = m_slicedBarModels.value(barSeries);
@@ -2026,7 +2012,7 @@ void QQuickGraphsBars::createSliceView()
                     slicedBarListSize = newRowSize;
 
                 for (int ind = 0; ind < slicedBarListSize; ++ind) {
-                    QQuick3DModel *model = createDataItem(sliceParent->scene(), barSeries);
+                    QQuick3DModel *model = createDataItem(sliceView()->scene(), barSeries);
                     model->setVisible(false);
                     BarModel *barModel = new BarModel();
                     barModel->model = model;
@@ -2045,10 +2031,10 @@ void QQuickGraphsBars::createSliceView()
                 }
 
                 if (barInstancing->selectedModel == nullptr) {
-                    barInstancing->selectedModel = createDataItem(sliceParent->scene(), barSeries);
+                    barInstancing->selectedModel = createDataItem(sliceView()->scene(), barSeries);
                     barInstancing->selectedModel->setInstancing(barInstancing->selectionInstancing);
                     barInstancing->selectedModel->setVisible(false);
-                    barInstancing->multiSelectedModel = createDataItem(sliceParent->scene(),
+                    barInstancing->multiSelectedModel = createDataItem(sliceView()->scene(),
                                                                        barSeries);
                     barInstancing->multiSelectedModel->setInstancing(
                         barInstancing->multiSelectionInstancing);
@@ -2336,31 +2322,27 @@ void QQuickGraphsBars::createBarItemHolders(QBar3DSeries *series,
 void QQuickGraphsBars::updateSelectionMode(QAbstract3DGraph::SelectionFlags mode)
 {
     checkSliceEnabled();
-    if (mode.testFlag(QAbstract3DGraph::SelectionSlice) && m_selectedBarSeries) {
-        setSliceActivatedChanged(true);
-        m_selectionDirty = !(sliceView()->isVisible());
-    } else {
-        if (sliceView() && sliceView()->isVisible()) {
+    if (!sliceView())
+        createSliceView();
+
+    bool validSlice = mode.testFlag(QAbstract3DGraph::SelectionSlice) && m_selectedBarSeries;
+    if (sliceView() && sliceView()->isVisible()) {
+        if (validSlice) {
+            removeSlicedBarModels();
+            createSliceView();
+            updateSliceGraph();
+        } else {
             m_selectionDirty = true;
             setSliceActivatedChanged(true);
         }
-    }
-
-    if (optimizationHint() == QAbstract3DGraph::OptimizationHint::Default) {
-        for (const auto barList : std::as_const(m_barModelsMap)) {
-            QList<BarItemHolder *> barItemList = barList->at(0)->instancing->dataArray();
-            for (auto bih : barItemList)
-                bih->selectedBar = false;
-            barList->at(0)->selectedModel->setVisible(false);
-            barList->at(0)->multiSelectedModel->setVisible(false);
-        }
-        if (sliceView() && sliceView()->isVisible())
-            removeSlicedBarModels();
+    } else if (validSlice) {
+        m_selectionDirty = true;
+        setSliceActivatedChanged(true);
     }
 
     setSeriesVisualsDirty(true);
     itemLabel()->setVisible(false);
-    if (sliceView() && sliceView()->isVisible())
+    if (sliceView() && !mode.testFlag(QAbstract3DGraph::SelectionItem))
         sliceItemLabel()->setVisible(false);
 }
 
