@@ -63,8 +63,10 @@ void tst_theme::construct()
     QCOMPARE(theme->colorStyle(), Q3DTheme::ColorStyle::Uniform);
     QCOMPARE(theme->font(), QFont("Arial"));
     QCOMPARE(theme->isGridEnabled(), true);
+    QCOMPARE(theme->isShaderGridEnabled(), false);
     QCOMPARE(theme->gridLineColor(), QColor(QRgb(0x35322f)));
-    QCOMPARE(theme->highlightLightStrength(), 5.0f);
+    QCOMPARE(theme->subgridLineColor(), QColor(QRgb(0x35322f)));
+    QCOMPARE(theme->gridWidth(), 0.25f);
     QCOMPARE(theme->labelBackgroundColor(), QColor(0x00, 0x00, 0x00, 0xcd));
     QCOMPARE(theme->isLabelBackgroundEnabled(), true);
     QCOMPARE(theme->isLabelBorderEnabled(), false);
@@ -96,8 +98,10 @@ void tst_theme::initialProperties()
     QCOMPARE(m_theme->colorStyle(), Q3DTheme::ColorStyle::Uniform);
     QCOMPARE(m_theme->font(), QFont());
     QCOMPARE(m_theme->isGridEnabled(), true);
+    QCOMPARE(m_theme->isShaderGridEnabled(), false);
     QCOMPARE(m_theme->gridLineColor(), QColor(Qt::white));
-    QCOMPARE(m_theme->highlightLightStrength(), 7.5f);
+    QCOMPARE(m_theme->subgridLineColor(), QColor(Qt::white));
+    QCOMPARE(m_theme->gridWidth(), 0.25f);
     QCOMPARE(m_theme->labelBackgroundColor(), QColor(Qt::gray));
     QCOMPARE(m_theme->isLabelBackgroundEnabled(), true);
     QCOMPARE(m_theme->isLabelBorderEnabled(), true);
@@ -137,8 +141,10 @@ void tst_theme::initializeProperties()
     m_theme->setColorStyle(Q3DTheme::ColorStyle::RangeGradient);
     m_theme->setFont(QFont("Arial"));
     m_theme->setGridEnabled(false);
+    m_theme->setShaderGridEnabled(true);
     m_theme->setGridLineColor(QColor(Qt::green));
-    m_theme->setHighlightLightStrength(5.0f);
+    m_theme->setSubgridLineColor(QColor(Qt::red));
+    m_theme->setGridWidth(0.8f);
     m_theme->setLabelBackgroundColor(QColor(Qt::gray));
     m_theme->setLabelBackgroundEnabled(false);
     m_theme->setLabelBorderEnabled(false);
@@ -164,8 +170,10 @@ void tst_theme::initializeProperties()
     QCOMPARE(m_theme->colorStyle(), Q3DTheme::ColorStyle::RangeGradient);
     QCOMPARE(m_theme->font(), QFont("Arial"));
     QCOMPARE(m_theme->isGridEnabled(), false);
+    QCOMPARE(m_theme->isShaderGridEnabled(), true);
     QCOMPARE(m_theme->gridLineColor(), QColor(Qt::green));
-    QCOMPARE(m_theme->highlightLightStrength(), 5.0f);
+    QCOMPARE(m_theme->subgridLineColor(), QColor(Qt::red));
+    QCOMPARE(m_theme->gridWidth(), 0.8f);
     QCOMPARE(m_theme->labelBackgroundColor(), QColor(Qt::gray));
     QCOMPARE(m_theme->isLabelBackgroundEnabled(), false);
     QCOMPARE(m_theme->isLabelBorderEnabled(), false);
@@ -188,11 +196,6 @@ void tst_theme::invalidProperties()
     m_theme->setAmbientLightStrength(1.1f);
     QCOMPARE(m_theme->ambientLightStrength(), 0.25f);
 
-    m_theme->setHighlightLightStrength(-1.0f);
-    QCOMPARE(m_theme->highlightLightStrength(), 7.5f);
-    m_theme->setHighlightLightStrength(10.1f);
-    QCOMPARE(m_theme->highlightLightStrength(), 7.5f);
-
     m_theme->setLightStrength(-1.0f);
     QCOMPARE(m_theme->lightStrength(), 5.0f);
     m_theme->setLightStrength(10.1f);
@@ -202,6 +205,11 @@ void tst_theme::invalidProperties()
     QCOMPARE(m_theme->shadowStrength(), 25.0f);
     m_theme->setShadowStrength(100.1f);
     QCOMPARE(m_theme->shadowStrength(), 25.0f);
+
+    m_theme->setGridWidth(2.0f);
+    QCOMPARE(m_theme->gridWidth(), 0.25f);
+    m_theme->setGridWidth(-0.2f);
+    QCOMPARE(m_theme->gridWidth(), 0.25f);
 }
 
 QTEST_MAIN(tst_theme)
