@@ -2,14 +2,12 @@
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include "chart.h"
-#include "custominputhandler.h"
 #include <QtGraphs/qcategory3daxis.h>
 #include <QtGraphs/qvalue3daxis.h>
 #include <QtGraphs/qlogvalue3daxisformatter.h>
 #include <QtGraphs/qbardataproxy.h>
 #include <QtGraphs/q3dscene.h>
 #include <QtGraphs/q3dtheme.h>
-#include <QtGraphs/q3dinputhandler.h>
 #include <QtGraphs/qcustom3ditem.h>
 #include <QtCore/QRandomGenerator>
 #include <QtCore/QElapsedTimer>
@@ -54,10 +52,8 @@ GraphModifier::GraphModifier(Q3DBars *barchart, QColorDialog *colorDialog)
       m_currentAxis(m_fixedRangeAxis),
       m_negativeValuesOn(false),
       m_useNullInputHandler(false),
-      m_defaultInputHandler(0),
       m_ownTheme(0),
       m_builtinTheme(new Q3DTheme(Q3DTheme::Theme::StoneMoss)),
-      m_customInputHandler(new CustomInputHandler),
       m_extraSeries(0)
 {
     m_temperatureData->setObjectName("m_temperatureData");
@@ -142,7 +138,7 @@ GraphModifier::GraphModifier(Q3DBars *barchart, QColorDialog *colorDialog)
     m_dummyData4->setItemLabelFormat(QStringLiteral("@seriesName: @valueLabel"));
     m_dummyData5->setItemLabelFormat(QStringLiteral("@seriesName: @valueLabel"));
 
-    m_genericData->dataProxy()->setColumnLabels(genericColumnLabels);
+    m_genericData->setColumnLabels(genericColumnLabels);
 
     m_temperatureData->setBaseColor(Qt::red);
     m_temperatureData->setSingleHighlightColor(Qt::cyan);
@@ -189,9 +185,8 @@ GraphModifier::GraphModifier(Q3DBars *barchart, QColorDialog *colorDialog)
     m_graph->activeTheme()->setFont(QFont("Times Roman", 20));
 
     // Release and store the default input handler.
-    m_defaultInputHandler = static_cast<Q3DInputHandler *>(m_graph->activeInputHandler());
-    m_graph->releaseInputHandler(m_defaultInputHandler);
-    m_graph->setActiveInputHandler(m_defaultInputHandler);
+    m_graph->unsetDefaultWheelHandler();
+    m_graph->setDefaultInputHandler();
 
     QObject::connect(m_graph, &Q3DBars::shadowQualityChanged, this,
                      &GraphModifier::shadowQualityUpdatedByVisual);
@@ -233,7 +228,6 @@ GraphModifier::GraphModifier(Q3DBars *barchart, QColorDialog *colorDialog)
 GraphModifier::~GraphModifier()
 {
     delete m_graph;
-    delete m_defaultInputHandler;
 }
 
 void GraphModifier::start()
@@ -348,19 +342,19 @@ void GraphModifier::createMassiveArray()
             genericRowLabels << QStringLiteral("Row %1").arg(i);
     }
 
-    QBarDataArray *dataArray = new QBarDataArray;
-    dataArray->reserve(arrayDimension);
+    QBarDataArray dataArray;
+    dataArray.reserve(arrayDimension);
     for (int i = 0; i < arrayDimension; i++) {
-        QBarDataRow *dataRow = new QBarDataRow(arrayDimension);
+        QBarDataRow dataRow(arrayDimension);
         for (int j = 0; j < arrayDimension; j++) {
             if (!m_negativeValuesOn)
-                (*dataRow)[j].setValue((float(i % 300 + 1) / 300.0)
+                dataRow[j].setValue((float(i % 300 + 1) / 300.0)
                                        * QRandomGenerator::global()->bounded(m_maxval));
             else
-                (*dataRow)[j].setValue((float(i % 300 + 1) / 300.0)
+                dataRow[j].setValue((float(i % 300 + 1) / 300.0)
                                        * QRandomGenerator::global()->bounded(m_maxval) + m_minval);
         }
-        dataArray->append(dataRow);
+        dataArray.append(dataRow);
     }
 
     m_genericData->dataProxy()->resetArray(dataArray, genericRowLabels, genericColumnLabels);
@@ -382,36 +376,35 @@ void GraphModifier::resetTemperatureData()
     };
 
     // Create data rows
-    QBarDataArray *dataSet = new QBarDataArray;
-    QBarDataRow *dataRow;
+    QBarDataArray dataSet;
 
-    dataSet->reserve(m_years.size());
+    dataSet.reserve(m_years.size());
     for (int year = 0; year < m_years.size(); year++) {
-        dataRow = new QBarDataRow(m_months.size());
+        QBarDataRow dataRow(m_months.size());
         // Create data items
         for (int month = 0; month < m_months.size(); month++) {
             // Add data to rows
-            (*dataRow)[month].setValue(temp[year][month]);
+            dataRow[month].setValue(temp[year][month]);
         }
         // Add row to set
-        dataSet->append(dataRow);
+        dataSet.append(dataRow);
     }
 
-    QBarDataArray *dataSet2 = new QBarDataArray;
+    QBarDataArray dataSet2;
 
-    dataSet2->reserve(m_years.size());
+    dataSet2.reserve(m_years.size());
     for (int year = m_years.size() - 1; year >= 0; year--) {
-        dataRow = new QBarDataRow(m_months.size());
+        QBarDataRow dataRow(m_months.size());
         // Create data items
         for (int month = m_months.size() - 1; month >= 0 ; month--) {
             // Add data to rows
-            (*dataRow)[month].setValue(temp[year][month]);
+            dataRow[month].setValue(temp[year][month]);
         }
         // Add row to set
-        dataSet2->append(dataRow);
+        dataSet2.append(dataRow);
     }
 
-    // Add data to chart (chart assumes ownership)
+    // Add data to graph (graph assumes ownership)
     m_temperatureData->dataProxy()->resetArray(dataSet, m_years, m_months);
     m_temperatureData2->dataProxy()->resetArray(dataSet2, m_years, m_months);
 }
@@ -423,13 +416,13 @@ static int changeCounter = 0;
 
 void GraphModifier::addRow()
 {
-    QBarDataRow *dataRow = new QBarDataRow(m_columnCount);
+    QBarDataRow dataRow(m_columnCount);
     for (float i = 0; i < m_columnCount; i++) {
         if (!m_negativeValuesOn)
-            (*dataRow)[i].setValue(((i + 1) / (float)m_columnCount)
+            dataRow[i].setValue(((i + 1) / (float)m_columnCount)
                                    * QRandomGenerator::global()->bounded(m_maxval));
         else
-            (*dataRow)[i].setValue(((i + 1) / (float)m_columnCount)
+            dataRow[i].setValue(((i + 1) / (float)m_columnCount)
                                    * QRandomGenerator::global()->bounded(m_maxval)
                                    - QRandomGenerator::global()->bounded(m_minval));
     }
@@ -444,9 +437,9 @@ void GraphModifier::addRows()
     QBarDataArray dataArray;
     QStringList labels;
     for (int i = 0; i < m_rowCount; i++) {
-        QBarDataRow *dataRow = new QBarDataRow(m_columnCount);
+        QBarDataRow dataRow(m_columnCount);
         for (int j = 0; j < m_columnCount; j++)
-            (*dataRow)[j].setValue(float(j + i + m_genericData->dataProxy()->rowCount()) + m_minval);
+            dataRow[j].setValue(float(j + i + m_genericData->dataProxy()->rowCount()) + m_minval);
         dataArray.append(dataRow);
         labels.append(QStringLiteral("Add %1").arg(addCounter++));
     }
@@ -457,9 +450,9 @@ void GraphModifier::addRows()
 
 void GraphModifier::insertRow()
 {
-    QBarDataRow *dataRow = new QBarDataRow(m_columnCount);
+    QBarDataRow dataRow(m_columnCount);
     for (float i = 0; i < m_columnCount; i++)
-        (*dataRow)[i].setValue(((i + 1) / (float)m_columnCount)
+        dataRow[i].setValue(((i + 1) / (float)m_columnCount)
                                * QRandomGenerator::global()->bounded(m_maxval) + m_minval);
 
     // TODO Needs to be changed to account for data window offset once it is implemented.
@@ -475,9 +468,9 @@ void GraphModifier::insertRows()
     QBarDataArray dataArray;
     QStringList labels;
     for (int i = 0; i < m_rowCount; i++) {
-        QBarDataRow *dataRow = new QBarDataRow(m_columnCount);
+        QBarDataRow dataRow(m_columnCount);
         for (int j = 0; j < m_columnCount; j++)
-            (*dataRow)[j].setValue(float(j + i + m_genericData->dataProxy()->rowCount()) + m_minval);
+            dataRow[j].setValue(float(j + i + m_genericData->dataProxy()->rowCount()) + m_minval);
         dataArray.append(dataRow);
         labels.append(QStringLiteral("Insert %1").arg(insertCounter++));
     }
@@ -504,9 +497,9 @@ void GraphModifier::changeRow()
     // TODO Needs to be changed to account for data window offset once it is implemented.
     int row = m_selectedBar.x();
     if (row >= 0) {
-        QBarDataRow *newRow = new QBarDataRow(m_genericData->dataProxy()->rowAt(row)->size());
-        for (int i = 0; i < newRow->size(); i++)
-            (*newRow)[i].setValue(QRandomGenerator::global()->bounded(m_maxval) + m_minval);
+        QBarDataRow newRow(m_genericData->dataProxy()->rowAt(row).size());
+        for (int i = 0; i < newRow.size(); i++)
+            newRow[i].setValue(QRandomGenerator::global()->bounded(m_maxval) + m_minval);
         QString label = QStringLiteral("Change %1").arg(changeCounter++);
         m_genericData->dataProxy()->setRow(row, newRow, label);
     }
@@ -521,9 +514,9 @@ void GraphModifier::changeRows()
         QBarDataArray newArray;
         QStringList labels;
         for (int i = startRow; i <= row; i++ ) {
-            QBarDataRow *newRow = new QBarDataRow(m_genericData->dataProxy()->rowAt(i)->size());
-            for (int j = 0; j < newRow->size(); j++)
-                (*newRow)[j].setValue(QRandomGenerator::global()->bounded(m_maxval) + m_minval);
+            QBarDataRow newRow(m_genericData->dataProxy()->rowAt(i).size());
+            for (int j = 0; j < newRow.size(); j++)
+                newRow[j].setValue(QRandomGenerator::global()->bounded(m_maxval) + m_minval);
             newArray.append(newRow);
             labels.append(QStringLiteral("Change %1").arg(changeCounter++));
         }
@@ -717,10 +710,13 @@ void GraphModifier::setUseNullInputHandler(int useNull)
 
     m_useNullInputHandler = useNull;
 
-    if (useNull)
-        m_graph->setActiveInputHandler(0);
-    else
-        m_graph->setActiveInputHandler(m_defaultInputHandler);
+    if (useNull) {
+        m_graph->unsetDefaultInputHandler();
+        QObject::disconnect(m_graph, &QAbstract3DGraph::wheel, this, &GraphModifier::onWheel);
+        QObject::disconnect(m_graph, &QAbstract3DGraph::mouseMove, this, &GraphModifier::onMouseMove);
+    } else {
+        m_graph->setDefaultInputHandler();
+    }
 }
 
 void GraphModifier::handleRowAxisChanged(QCategory3DAxis *axis)
@@ -756,7 +752,7 @@ void GraphModifier::showFiveSeries()
     releaseAxes();
     m_graph->setSelectionMode(QAbstract3DGraph::SelectionItemRowAndColumn | QAbstract3DGraph::SelectionMultiSeries);
 
-    m_dummyData->dataProxy()->resetArray(makeDummyData(), QStringList(), QStringList());
+    m_dummyData->dataProxy()->resetArray(makeDummyData());
     m_dummyData2->dataProxy()->resetArray(makeDummyData(), QStringList(), QStringList());
     m_dummyData3->dataProxy()->resetArray(makeDummyData(), QStringList(), QStringList());
     m_dummyData4->dataProxy()->resetArray(makeDummyData(), QStringList(), QStringList());
@@ -773,7 +769,7 @@ void GraphModifier::showFiveSeries()
         m_graph->addSeries(m_dummyData5);
 }
 
-QBarDataArray *GraphModifier::makeDummyData()
+QBarDataArray GraphModifier::makeDummyData()
 {
     // Set up data
     static const float temp[4][4] = {
@@ -784,19 +780,19 @@ QBarDataArray *GraphModifier::makeDummyData()
     };
 
     // Create data rows
-    QBarDataArray *dataSet = new QBarDataArray;
-    QBarDataRow *dataRow;
+    QBarDataArray dataSet;
+    QBarDataRow dataRow;
 
-    dataSet->reserve(4);
+    dataSet.reserve(4);
     for (int i = 0; i < 4; i++) {
-        dataRow = new QBarDataRow(4);
+        dataRow.resize(4);
         // Create data items
         for (int j = 0; j < 4; j++) {
             // Add data to rows
-            (*dataRow)[j].setValue(temp[i][j]);
+            dataRow[j].setValue(temp[i][j]);
         }
         // Add row to set
-        dataSet->append(dataRow);
+        dataSet.append(dataRow);
     }
     return dataSet;
 }
@@ -1010,13 +1006,17 @@ void GraphModifier::insertRemoveTestToggle()
         m_graph->removeSeries(m_dummyData2);
         releaseSeries();
         releaseAxes();
-        m_graph->setActiveInputHandler(m_defaultInputHandler);
+        QObject::disconnect(m_graph, &QAbstract3DGraph::wheel, this, &GraphModifier::onWheel);
+        QObject::disconnect(m_graph, &QAbstract3DGraph::mouseMove, this, &GraphModifier::onMouseMove);
+        m_graph->setDefaultInputHandler();
     } else {
         releaseSeries();
         releaseAxes();
         m_graph->rowAxis()->setRange(0, 32);
         m_graph->columnAxis()->setRange(0, 10);
-        m_graph->setActiveInputHandler(m_customInputHandler);
+        m_graph->unsetDefaultWheelHandler();
+        QObject::connect(m_graph, &QAbstract3DGraph::wheel, this, &GraphModifier::onWheel);
+        QObject::connect(m_graph, &QAbstract3DGraph::mouseMove, this, &GraphModifier::onMouseMove);
         m_graph->addSeries(m_dummyData);
         m_graph->addSeries(m_dummyData2);
         m_insertRemoveStep = 0;
@@ -1145,12 +1145,12 @@ void GraphModifier::addRemoveSeries()
         break;
     case 1: {
         qDebug() << __FUNCTION__ << counter << "Add data to series";
-        QBarDataArray *array = new QBarDataArray;
-        array->reserve(5);
+        QBarDataArray array;
+        array.reserve(5);
         for (int i = 0; i < 5; i++) {
-            array->append(new QBarDataRow(10));
+            array.append(QBarDataRow(10));
             for (int j = 0; j < 10; j++)
-                (*array->at(i))[j].setValue(i * j);
+               array[i][j].setValue(i * j);
         }
         m_extraSeries->dataProxy()->resetArray(array);
     }
@@ -1165,9 +1165,9 @@ void GraphModifier::addRemoveSeries()
         QBarDataArray array;
         array.reserve(5);
         for (int i = 0; i < 5; i++) {
-            array.append(new QBarDataRow(10));
+            array.append(QBarDataRow(10));
             for (int j = 0; j < 10; j++)
-                (*array.at(i))[j].setValue(2 * i * j);
+                array[i][j].setValue(2 * i * j);
         }
         m_extraSeries->dataProxy()->addRows(array);
     }
@@ -1384,7 +1384,7 @@ void GraphModifier::testItemAndRowChanges(bool checked)
         qDebug() << __FUNCTION__ << counter << "Change row to different length";
         series0->dataProxy()->setRow(4, createFlatRow(5, 20.0f));
         series1->dataProxy()->setRow(4, createFlatRow(0, 20.0f));
-        series2->dataProxy()->setRow(4, 0);
+        series2->dataProxy()->setRow(4, QBarDataRow());
     }
         break;
     case 19: {
@@ -1406,22 +1406,22 @@ void GraphModifier::reverseValueAxis(int enabled)
 
 void GraphModifier::setInputHandlerRotationEnabled(int enabled)
 {
-    m_defaultInputHandler->setRotationEnabled(enabled);
+    m_graph->setRotationEnabled(enabled);
 }
 
 void GraphModifier::setInputHandlerZoomEnabled(int enabled)
 {
-    m_defaultInputHandler->setZoomEnabled(enabled);
+    m_graph->setZoomEnabled(enabled);
 }
 
 void GraphModifier::setInputHandlerSelectionEnabled(int enabled)
 {
-    m_defaultInputHandler->setSelectionEnabled(enabled);
+    m_graph->setSelectionEnabled(enabled);
 }
 
 void GraphModifier::setInputHandlerZoomAtTargetEnabled(int enabled)
 {
-    m_defaultInputHandler->setZoomAtTargetEnabled(enabled);
+    m_graph->setZoomAtTargetEnabled(enabled);
 }
 
 void GraphModifier::changeValueAxisSegments(int value)
@@ -1435,9 +1435,9 @@ void GraphModifier::insertRemoveTimerTimeout()
 {
     if (m_insertRemoveStep < 32) {
         for (int k = 0; k < 1; k++) {
-            QBarDataRow *dataRow = new QBarDataRow(10);
+            QBarDataRow dataRow(10);
             for (float i = 0; i < 10; i++)
-                (*dataRow)[i].setValue(((i + 1) / 10.0f) * (float)(QRandomGenerator::global()->bounded(100)));
+                dataRow[i].setValue(((i + 1) / 10.0f) * (float)(QRandomGenerator::global()->bounded(100)));
 
             QString label = QStringLiteral("Insert %1").arg(insertCounter++);
             m_dummyData->dataProxy()->insertRow(0, dataRow, label);
@@ -1449,9 +1449,9 @@ void GraphModifier::insertRemoveTimerTimeout()
 
     if (m_insertRemoveStep < 16 || (m_insertRemoveStep > 31 && m_insertRemoveStep < 48)) {
         for (int k = 0; k < 2; k++) {
-            QBarDataRow *dataRow = new QBarDataRow(10);
+            QBarDataRow dataRow(10);
             for (float i = 0; i < 10; i++)
-                (*dataRow)[i].setValue(((i + 1) / 10.0f) * (float)(QRandomGenerator::global()->bounded(100)));
+                dataRow[i].setValue(((i + 1) / 10.0f) * (float)(QRandomGenerator::global()->bounded(100)));
 
             QString label = QStringLiteral("Insert %1").arg(insertCounter++);
             m_dummyData2->dataProxy()->insertRow(0, dataRow, label);
@@ -1467,7 +1467,7 @@ void GraphModifier::insertRemoveTimerTimeout()
 
 void GraphModifier::triggerSelection()
 {
-    m_graph->scene()->setSelectionQueryPosition(m_customInputHandler->inputPosition());
+    m_graph->doPicking(m_mousePos);
 }
 
 void GraphModifier::triggerRotation()
@@ -1475,7 +1475,7 @@ void GraphModifier::triggerRotation()
     if (m_selectedSeries) {
         QPoint selectedBar = m_selectedSeries->selectedBar();
         if (selectedBar != QBar3DSeries::invalidSelectionPosition()) {
-            QBarDataItem item(*(m_selectedSeries->dataProxy()->itemAt(selectedBar.x(), selectedBar.y())));
+            QBarDataItem item(m_selectedSeries->dataProxy()->itemAt(selectedBar.x(), selectedBar.y()));
             item.setRotation(item.rotation() + 1.0f);
             m_selectedSeries->dataProxy()->setItem(selectedBar.x(), selectedBar.y(), item);
         }
@@ -1534,15 +1534,38 @@ void GraphModifier::setGraphMargin(int value)
     qDebug() << "Setting margin:" << m_graph->margin() << value;
 }
 
+void GraphModifier::onWheel(QWheelEvent *event)
+{
+    int zoomLevel = m_graph->cameraZoomLevel();
+    if (zoomLevel > 100)
+        zoomLevel += event->angleDelta().y() / 12;
+    else if (zoomLevel > 50)
+        zoomLevel += event->angleDelta().y() / 60;
+    else
+        zoomLevel += event->angleDelta().y() / 120;
+    if (zoomLevel > 500)
+        zoomLevel = 500;
+    else if (zoomLevel < 10)
+        zoomLevel = 10;
+
+    m_graph->setCameraZoomLevel(zoomLevel);
+}
+
+void GraphModifier::onMouseMove(QPoint mousePos)
+{
+    m_mousePos = mousePos;
+    m_graph->doPicking(mousePos);
+}
+
 void GraphModifier::populateFlatSeries(QBar3DSeries *series, int rows, int columns, float value)
 {
-    QBarDataArray *dataArray = new QBarDataArray;
-    dataArray->reserve(rows);
+    QBarDataArray dataArray;
+    dataArray.reserve(rows);
     for (int i = 0; i < rows; i++) {
-        QBarDataRow *dataRow = new QBarDataRow(columns);
+        QBarDataRow dataRow(columns);
         for (int j = 0; j < columns; j++)
-            (*dataRow)[j].setValue(value);
-        dataArray->append(dataRow);
+            dataRow[j].setValue(value);
+        dataArray.append(dataRow);
     }
     QStringList axisLabels;
     int count = qMax(rows, columns);
@@ -1552,11 +1575,11 @@ void GraphModifier::populateFlatSeries(QBar3DSeries *series, int rows, int colum
     series->dataProxy()->resetArray(dataArray, axisLabels, axisLabels);
 }
 
-QBarDataRow *GraphModifier::createFlatRow(int columns, float value)
+QBarDataRow GraphModifier::createFlatRow(int columns, float value)
 {
-    QBarDataRow *dataRow = new QBarDataRow(columns);
+    QBarDataRow dataRow(columns);
     for (int j = 0; j < columns; j++)
-        (*dataRow)[j].setValue(value);
+        dataRow[j].setValue(value);
     return dataRow;
 }
 
@@ -1677,7 +1700,6 @@ void GraphModifier::useOwnTheme(bool checked)
         m_ownTheme->setBaseColors(colors);
         m_ownTheme->setColorStyle(Q3DTheme::ColorStyle::Uniform);
         m_ownTheme->setGridLineColor(QColor(QRgb(0x99ca53)));
-        m_ownTheme->setHighlightLightStrength(7.0f);
         m_ownTheme->setLabelBackgroundEnabled(true);
         m_ownTheme->setLabelBorderEnabled(true);
         m_ownTheme->setLightColor(Qt::white);
@@ -1743,17 +1765,6 @@ void GraphModifier::toggleMultiseriesScaling()
     m_graph->setMultiSeriesUniform(!m_graph->isMultiSeriesUniform());
 }
 
-void GraphModifier::setReflection(int enabled)
-{
-    m_graph->setReflection(enabled);
-}
-
-void GraphModifier::setReflectivity(int value)
-{
-    qreal reflectivity = (qreal)value / 100.0;
-    m_graph->setReflectivity(reflectivity);
-}
-
 void GraphModifier::toggleCustomItem()
 {
     static int counter = 0;
